<?php
/**
 * @version 1.0 $Id$
 * @package Joomla
 * @subpackage FLEXIcontent
 * @subpackage plugin.content.ggstreetview
 * @copyright (C) 2014 ColorPack Creation Co.,Ltd. - www.colorpack.co.th
 * @license GNU/GPL v3
 *
 * FLEXIcontent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
defined( '_JEXEC' ) or die( 'Restricted access' );

//jimport('joomla.plugin.plugin');
jimport('joomla.event.plugin');

class PlgContentGGStreetView extends JPlugin {
	/**
	 * Load the language file on instantiation.
	 *
	 * @var    boolean
	 * @since  3.1
	 */
	protected $autoloadLanguage = true;
	
	/**
	 * Plugin that adds a ggstreetview into the text and truncates text at that point
	 *
	 * @param   string   $context  The context of the content being passed to the plugin.
	 * @param   object   &$row     The article object.  Note $article->text is also available
	 * @param   mixed    &$params  The article params
	 * @param   integer  $page     The 'page' number
	 *
	 * @return  mixed  Always returns void or true
	 *
	 * @since   1.6
	 */
	public function onContentPrepare($context, &$row, &$params, $page = 0)
	{
		$canProceed = $context == 'com_content.article';
		if (!$canProceed) {
			return;
		}

		// Expression to search for.
		$regex = '#\{streetview(.*)\}([0-9\-\.\s,]+)\{\/streetview\}#iU';

		$input = JFactory::getApplication()->input;

		$print = $input->getBool('print');

		if (!$this->params->get('enabled', 1))
		{
			$print = true;
		}

		if ($print)
		{
			$row->text = preg_replace($regex, '<br />', $row->text);

			return true;
		}

		/*// Simple performance check to determine whether bot should process further.
		if (JString::strpos($row->text, 'class="system-pagebreak') === false)
		{
			return true;
		}*/

		$view = $input->getString('view');
		$full = $input->getBool('fullview');

		/*if ($params->get('intro_only') || $params->get('popup') || $full || $view != 'article')
		{
			$row->text = preg_replace($regex, '', $row->text);

			return;
		}*/

		// Find all instances of plugin and put in $matches.
		$matches = array();
		preg_match_all($regex, $row->text, $matches, PREG_SET_ORDER);
		$js = "function initialize".$row->id."() {\r\n";
		$text = '';
		foreach($matches as $k=>$match) {
			$streetview_params = $this->parseParams($match[1]);
			$streetview_params->h = @$streetview_params->h?$streetview_params->h:350;
			$streetview_params->w = @$streetview_params->w?$streetview_params->w:500;
			$values = $this->parseValues($match[2]);
			$js .= "var mapposition{$k} = new google.maps.LatLng(".$values['lat'].", ".$values['lon'].");

			// Note: constructed panorama objects have visible: true
			// set by default.
			var panoOptions{$k} = {
				position: mapposition{$k},
				addressControlOptions: {
				  position: google.maps.ControlPosition.BOTTOM_CENTER
				},
				linksControl: false,
				panControl: false,
				zoomControlOptions: {
				  style: google.maps.ZoomControlStyle.SMALL
				},
				enableCloseButton: false
			};
			var panorama = new google.maps.StreetViewPanorama(
			document.getElementById('map-canvas-".$k."'), panoOptions{$k});\r\n";
			
			$text .= '<div id="map-canvas-'.$k.'" style="width: '.$streetview_params->w.'px; height: '.$streetview_params->h.'px"></div>';
		}
		$js .= "}
		google.maps.event.addDomListener(window, 'load', initialize".$row->id.");";
		
		$row->text = preg_replace($regex, $text, $row->text);
		
		$doc = JFactory::getDocument();
		if(!@$doc->google_maps_api) {
			$doc->addScript('https://maps.googleapis.com/maps/api/js?v=3.exp');
			$doc->google_maps_api = 1;
		}
		$doc->addScriptDeclaration($js);

		return true;
	}
	
	protected function parseParams($str) {
		$params = array();
		$a_s = explode(' ', $str);
		foreach($a_s as $s) {
			$s = trim($s);
			if(strlen($s)<=0) continue;
			$attrs = $this->parseAttributes($s);
			$params = array_merge($params, $attrs);
		}
		return (object)$params;
	}
	
	protected function parseValues($str) {
		$a_s = explode(',', $str);
		$values = array();
		$keys = array(0=>'lat',1=>'lon');
		foreach($keys as $kk=>$k) {
			$values[$k] = $a_s[$kk];
		}
		return $values;
	}
	
	protected function parseAttributes($str) {
		$pattern = '/(\\w+)\s*=\\s*("[^"]*"|\'[^\']*\'|[^"\'\\s>]*)/';
		$matches = array();
		preg_match_all($pattern, $str, $matches, PREG_SET_ORDER);
		$attrs = array();
		foreach ($matches as $match) {
			if (($match[2][0] == '"' || $match[2][0] == "'") && $match[2][0] == $match[2][strlen($match[2])-1]) {
				$match[2] = substr($match[2], 1, -1);
			}
			$name = strtolower($match[1]);
			$value = html_entity_decode($match[2]);
			switch ($name) {
			case 'class':
				$attrs[$name] = preg_split('/\s+/', trim($value));
				break;
			case 'style':
				// parse CSS property declarations
				break;
			default:
				$attrs[$name] = $value;
			}
		}
		return $attrs;
	}
}